@@ -9,8 +9,9 @@ Created by kevincar <kevincarrolldavis@gmail.com>
 import asyncio
 import itertools
 import logging
-from typing import Callable, Any, Dict, Iterable, NewType, Optional
+from typing import Any, Dict, Iterable, NewType, Optional
 
+import async_timeout
 import objc
 from Foundation import NSNumber, NSObject, NSArray, NSData, NSError, NSUUID, NSString
 from CoreBluetooth import (
@@ -21,7 +22,8 @@ from CoreBluetooth import (
     CBCharacteristicWriteWithResponse,
 )
 
-from bleak.exc import BleakError
+from ...exc import BleakError
+from ..client import NotifyCallback
 
 # logging.basicConfig(level=logging.DEBUG)
 logger = logging.getLogger(__name__)
@@ -59,7 +61,7 @@ class PeripheralDelegate(NSObject):
         self._descriptor_write_futures: Dict[int, asyncio.Future] = {}
 
         self._characteristic_notify_change_futures: Dict[int, asyncio.Future] = {}
-        self._characteristic_notify_callbacks: Dict[int, Callable[[str, Any], Any]] = {}
+        self._characteristic_notify_callbacks: Dict[int, NotifyCallback] = {}
 
         self._read_rssi_futures: Dict[NSUUID, asyncio.Future] = {}
 
@@ -134,7 +136,7 @@ class PeripheralDelegate(NSObject):
         self,
         characteristic: CBCharacteristic,
         use_cached: bool = True,
-        timeout: int = 10,
+        timeout: int = 20,
     ) -> NSData:
         if characteristic.value() is not None and use_cached:
             return characteristic.value()
@@ -144,7 +146,8 @@ class PeripheralDelegate(NSObject):
         self._characteristic_read_futures[characteristic.handle()] = future
         try:
             self.peripheral.readValueForCharacteristic_(characteristic)
-            return await asyncio.wait_for(future, timeout=timeout)
+            async with async_timeout.timeout(timeout):
+                return await future
         finally:
             del self._characteristic_read_futures[characteristic.handle()]
 
@@ -202,7 +205,7 @@ class PeripheralDelegate(NSObject):
 
     @objc.python_method
     async def start_notifications(
-        self, characteristic: CBCharacteristic, callback: Callable[[str, Any], Any]
+        self, characteristic: CBCharacteristic, callback: NotifyCallback
     ) -> None:
         c_handle = characteristic.handle()
         if c_handle in self._characteristic_notify_callbacks:
@@ -364,8 +367,9 @@ class PeripheralDelegate(NSObject):
         if not future:
             if error is None:
                 notify_callback = self._characteristic_notify_callbacks.get(c_handle)
+
                 if notify_callback:
-                    notify_callback(c_handle, bytearray(value))
+                    notify_callback(bytearray(value))
             return
 
         if error is not None:
